"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AdminClient = void 0;
const workflows_1 = require("../../protoc/workflows");
const hatchet_error_1 = __importDefault(require("../../util/errors/hatchet-error"));
const logger_1 = require("../../util/logger");
const retrier_1 = require("../../util/retrier");
const workflow_run_ref_1 = __importDefault(require("../../util/workflow-run-ref"));
/**
 * AdminClient is a client for interacting with the Hatchet Admin API. This allows you to configure, trigger,
 * and monitor workflows.
 * The admin client can be accessed via:
 * ```typescript
 * const hatchet = Hatchet.init()
 * const admin = hatchet.admin as AdminClient;
 *
 * // Now you can use the admin client to interact with the Hatchet Admin API
 * admin.list_workflows().then((res) => {
 *   res.rows?.forEach((row) => {
 *     console.log(row);
 *   });
 * });
 * ```
 */
class AdminClient {
    constructor(config, channel, factory, api, tenantId, listenerClient) {
        this.config = config;
        this.client = factory.create(workflows_1.WorkflowServiceDefinition, channel);
        this.api = api;
        this.tenantId = tenantId;
        this.logger = new logger_1.Logger(`Admin`, config.log_level);
        this.listenerClient = listenerClient;
    }
    /**
     * @deprecated use putWorkflow instead
     */
    put_workflow(opts) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.putWorkflow(opts);
        });
    }
    /**
     * Creates a new workflow or updates an existing workflow. If the workflow already exists, Hatchet will automatically
     * determine if the workflow definition has changed and create a new version if necessary.
     * @param workflow a workflow definition to create
     */
    putWorkflow(workflow) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield (0, retrier_1.retrier)(() => __awaiter(this, void 0, void 0, function* () { return this.client.putWorkflow({ opts: workflow }); }), this.logger);
            }
            catch (e) {
                throw new hatchet_error_1.default(e.message);
            }
        });
    }
    /**
     * @deprecated use putRateLimit instead
     */
    put_rate_limit(key, limit, duration) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.putRateLimit(key, limit, duration);
        });
    }
    putRateLimit(key_1, limit_1) {
        return __awaiter(this, arguments, void 0, function* (key, limit, duration = workflows_1.RateLimitDuration.SECOND) {
            try {
                yield (0, retrier_1.retrier)(() => __awaiter(this, void 0, void 0, function* () {
                    return this.client.putRateLimit({
                        key,
                        limit,
                        duration,
                    });
                }), this.logger);
            }
            catch (e) {
                throw new hatchet_error_1.default(e.message);
            }
        });
    }
    registerWebhook(data) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.api.webhookCreate(this.tenantId, data);
        });
    }
    /**
     * @deprecated use runWorkflow instead
     */
    run_workflow(workflowName, input, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.runWorkflow(workflowName, input, options);
        });
    }
    /**
     * Run a new instance of a workflow with the given input. This will create a new workflow run and return the ID of the
     * new run.
     * @param workflowName the name of the workflow to run
     * @param input an object containing the input to the workflow
     * @param options an object containing the options to run the workflow
     * @returns the ID of the new workflow run
     */
    runWorkflow(workflowName, input, options) {
        let computedName = workflowName;
        try {
            if (this.config.namespace && !workflowName.startsWith(this.config.namespace)) {
                computedName = this.config.namespace + workflowName;
            }
            const inputStr = JSON.stringify(input);
            const resp = this.client.triggerWorkflow(Object.assign(Object.assign({ name: computedName, input: inputStr }, options), { additionalMetadata: (options === null || options === void 0 ? void 0 : options.additionalMetadata)
                    ? JSON.stringify(options === null || options === void 0 ? void 0 : options.additionalMetadata)
                    : undefined }));
            return new workflow_run_ref_1.default(resp, this.listenerClient, options === null || options === void 0 ? void 0 : options.parentId);
        }
        catch (e) {
            throw new hatchet_error_1.default(e.message);
        }
    }
    /**
     * Run multiple workflows runs with the given input and options. This will create new workflow runs and return their IDs.
     * Order is preserved in the response.
     * @param workflowRuns an array of objects containing the workflow name, input, and options for each workflow run
     * @returns an array of workflow run references
     */
    runWorkflows(workflowRuns) {
        // Prepare workflows to be triggered in bulk
        const workflowRequests = workflowRuns.map(({ workflowName, input, options }) => {
            let computedName = workflowName;
            if (this.config.namespace && !workflowName.startsWith(this.config.namespace)) {
                computedName = this.config.namespace + workflowName;
            }
            const inputStr = JSON.stringify(input);
            return Object.assign(Object.assign({ name: computedName, input: inputStr }, options), { additionalMetadata: (options === null || options === void 0 ? void 0 : options.additionalMetadata)
                    ? JSON.stringify(options.additionalMetadata)
                    : undefined });
        });
        try {
            // Call the bulk trigger workflow method
            const bulkTriggerWorkflowResponse = this.client.bulkTriggerWorkflow(workflows_1.BulkTriggerWorkflowRequest.create({
                workflows: workflowRequests,
            }));
            return bulkTriggerWorkflowResponse.then((res) => {
                return res.workflowRunIds.map((resp, index) => {
                    const { options } = workflowRuns[index];
                    return new workflow_run_ref_1.default(resp, this.listenerClient, options === null || options === void 0 ? void 0 : options.parentId);
                });
            });
        }
        catch (e) {
            throw new hatchet_error_1.default(e.message);
        }
    }
    /**
     * @deprecated use listWorkflows instead
     */
    list_workflows() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.listWorkflows();
        });
    }
    /**
     * List workflows in the tenant associated with the API token.
     * @returns a list of all workflows in the tenant
     */
    listWorkflows() {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.api.workflowList(this.tenantId);
            return res.data;
        });
    }
    /**
     * @deprecated use getWorkflow instead
     */
    get_workflow(workflowId) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getWorkflow(workflowId);
        });
    }
    /**
     * Get a workflow by its ID.
     * @param workflowId the workflow ID (**note:** this is not the same as the workflow version id)
     * @returns
     */
    getWorkflow(workflowId) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.api.workflowGet(workflowId);
            return res.data;
        });
    }
    /**
     * @deprecated use getWorkflowVersion instead
     */
    get_workflow_version(workflowId, version) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getWorkflowVersion(workflowId, version);
        });
    }
    /**
     * Get a workflow version.
     * @param workflowId the workflow ID
     * @param version the version of the workflow to get. If not provided, the latest version will be returned.
     * @returns the workflow version
     */
    getWorkflowVersion(workflowId, version) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.api.workflowVersionGet(workflowId, {
                version,
            });
            return res.data;
        });
    }
    /**
     * @deprecated use getWorkflowRun instead
     */
    get_workflow_run(workflowRunId) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getWorkflowRun(workflowRunId);
        });
    }
    /**
     * Get a workflow run.
     * @param workflowRunId the id of the workflow run to get
     * @returns the workflow run
     */
    getWorkflowRun(workflowRunId) {
        return __awaiter(this, void 0, void 0, function* () {
            return new workflow_run_ref_1.default(workflowRunId, this.listenerClient);
        });
    }
    /**
     * @deprecated use listWorkflowRuns instead
     */
    list_workflow_runs(query) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.listWorkflowRuns(query);
        });
    }
    /**
     * List workflow runs in the tenant associated with the API token.
     * @param query the query to filter the list of workflow runs
     * @returns
     */
    listWorkflowRuns(query) {
        return __awaiter(this, void 0, void 0, function* () {
            const res = yield this.api.workflowRunList(this.tenantId, query);
            return res.data;
        });
    }
    /**
     * @deprecated use scheduleWorkflow instead
     */
    schedule_workflow(name, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.scheduleWorkflow(name, options);
        });
    }
    /**
     * Schedule a workflow to run at a specific time or times.
     * @param name the name of the workflow to schedule
     * @param options an object containing the schedules to set
     * @param input an object containing the input to the workflow
     */
    scheduleWorkflow(name, options) {
        let computedName = name;
        try {
            if (this.config.namespace && !name.startsWith(this.config.namespace)) {
                computedName = this.config.namespace + name;
            }
            let input;
            if (options === null || options === void 0 ? void 0 : options.input) {
                input = JSON.stringify(options.input);
            }
            this.client.scheduleWorkflow({
                name: computedName,
                schedules: options === null || options === void 0 ? void 0 : options.schedules,
                input,
            });
        }
        catch (e) {
            throw new hatchet_error_1.default(e.message);
        }
    }
    /**
     * @deprecated use getWorkflowMetrics instead
     */
    get_workflow_metrics(data) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.getWorkflowMetrics(data);
        });
    }
    /**
     * Get the metrics for a workflow.
     *
     * @param workflowId the ID of the workflow to get metrics for
     * @param workflowName the name of the workflow to get metrics for
     * @param query an object containing query parameters to filter the metrics
     */
    getWorkflowMetrics({ workflowId, workflowName, status, groupKey }) {
        const params = {
            status,
            groupKey,
        };
        if (workflowName) {
            this.listWorkflows().then((res) => {
                var _a;
                const workflow = (_a = res.rows) === null || _a === void 0 ? void 0 : _a.find((row) => row.name === workflowName);
                if (workflow) {
                    return this.api.workflowGetMetrics(workflow.metadata.id, params);
                }
                throw new Error(`Workflow ${workflowName} not found`);
            });
        }
        else if (workflowId) {
            return this.api.workflowGetMetrics(workflowId, params);
        }
        throw new Error('Must provide either a workflowId or workflowName');
    }
}
exports.AdminClient = AdminClient;
