"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebhookHandler = void 0;
const hatchet_error_1 = __importDefault(require("../../util/errors/hatchet-error"));
const crypto_1 = require("crypto");
const okMessage = 'The Hatchet webhooks endpoint is up and running!';
class WebhookHandler {
    // eslint-disable-next-line no-useless-constructor
    constructor(worker, workflows
    // eslint-disable-next-line no-empty-function
    ) {
        this.worker = worker;
        this.workflows = workflows;
    }
    /**
     * Handles a request with a provided body, secret, and signature.
     *
     * @param {string | undefined} body - The body of the request.
     * @param {string | undefined} secret - The secret used for signature verification.
     * @param {string | string[] | undefined | null} signature - The signature of the request.
     *
     * @throws {HatchetError} - If no signature is provided or the signature is not a string.
     * @throws {HatchetError} - If no secret is provided.
     * @throws {HatchetError} - If no body is provided.
     */
    handle(body, signature, secret) {
        return __awaiter(this, void 0, void 0, function* () {
            this.checkSignature(body, signature, secret);
            const action = JSON.parse(body);
            yield this.worker.handleAction(action);
        });
    }
    checkSignature(body, signature, secret) {
        if (!signature || typeof signature !== 'string') {
            throw new hatchet_error_1.default('No signature provided');
        }
        if (!secret) {
            throw new hatchet_error_1.default('No secret provided');
        }
        if (!body) {
            throw new hatchet_error_1.default('No body provided');
        }
        // verify hmac signature
        const actualSignature = (0, crypto_1.createHmac)('sha256', secret).update(body).digest('hex');
        if (actualSignature !== signature) {
            throw new hatchet_error_1.default(`Invalid signature, expected ${actualSignature}, got ${signature}`);
        }
    }
    getHealthcheckResponse(body, signature, secret) {
        return __awaiter(this, void 0, void 0, function* () {
            this.checkSignature(body, signature, secret);
            for (const workflow of this.workflows) {
                yield this.worker.registerWorkflow(workflow);
            }
            return {
                actions: Object.keys(this.worker.action_registry),
            };
        });
    }
    /**
     * Express Handler
     *
     * This method is an asynchronous function that returns an Express middleware handler.
     * The handler function is responsible for handling incoming requests and invoking the
     * corresponding logic based on the provided secret.
     */
    expressHandler({ secret }) {
        return (req, res) => {
            if (req.method === 'GET') {
                res.sendStatus(200);
                res.send(okMessage);
                return;
            }
            if (req.method === 'PUT') {
                this.getHealthcheckResponse(req.body, req.headers['x-hatchet-signature'], secret)
                    .then((resp) => {
                    res.sendStatus(200);
                    res.json(resp);
                })
                    .catch((err) => {
                    res.sendStatus(500);
                    this.worker.logger.error(`Error handling request: ${err.message}`);
                });
                return;
            }
            if (req.method !== 'POST') {
                res.sendStatus(405);
                res.json({ error: 'Method not allowed' });
                return;
            }
            this.handle(req.body, req.headers['x-hatchet-signature'], secret)
                .then(() => {
                res.sendStatus(200);
            })
                .catch((err) => {
                res.sendStatus(500);
                this.worker.logger.error(`Error handling request: ${err.message}`);
            });
        };
    }
    /**
     * A method that returns an HTTP request handler.
     */
    httpHandler({ secret }) {
        return (req, res) => {
            const handle = () => __awaiter(this, void 0, void 0, function* () {
                if (req.method === 'GET') {
                    res.writeHead(200, { 'Content-Type': 'application/json' });
                    res.write(okMessage);
                    res.end();
                    return;
                }
                const body = yield this.getBody(req);
                if (req.method === 'PUT') {
                    const resp = yield this.getHealthcheckResponse(body, req.headers['x-hatchet-signature'], secret);
                    res.writeHead(200, { 'Content-Type': 'application/json' });
                    res.write(JSON.stringify(resp));
                    res.end();
                    return;
                }
                if (req.method !== 'POST') {
                    res.writeHead(405, { 'Content-Type': 'application/json' });
                    res.write(JSON.stringify({ error: 'Method not allowed' }));
                    res.end();
                    return;
                }
                yield this.handle(body, req.headers['x-hatchet-signature'], secret);
                res.writeHead(200, 'OK');
                res.end();
            });
            handle().catch((e) => {
                this.worker.logger.error(`Error handling request: ${e.message}`);
                res.writeHead(500, 'Internal server error');
                res.end();
            });
        };
    }
    /**
     * A method that returns a Next.js pages router request handler.
     */
    nextJSPagesHandler({ secret }) {
        return (req, res) => __awaiter(this, void 0, void 0, function* () {
            if (req.method === 'GET') {
                return res.status(200).send(okMessage);
            }
            const sig = req.headers['x-hatchet-signature'];
            const body = JSON.stringify(req.body);
            if (req.method === 'PUT') {
                const resp = yield this.getHealthcheckResponse(body, sig, secret);
                return res.status(200).send(JSON.stringify(resp));
            }
            if (req.method !== 'POST') {
                return res.status(405).send('Method not allowed');
            }
            yield this.handle(body, sig, secret);
            return res.status(200).send('ok');
        });
    }
    /**
     * A method that returns a Next.js request handler.
     */
    nextJSHandler({ secret }) {
        const ok = () => __awaiter(this, void 0, void 0, function* () {
            return new Response(okMessage, { status: 200 });
        });
        const f = (req) => __awaiter(this, void 0, void 0, function* () {
            const sig = req.headers.get('x-hatchet-signature');
            const body = yield req.text();
            if (req.method === 'PUT') {
                const resp = yield this.getHealthcheckResponse(body, sig, secret);
                return new Response(JSON.stringify(resp), { status: 200 });
            }
            if (req.method !== 'POST') {
                return new Response('Method not allowed', { status: 405 });
            }
            yield this.handle(body, sig, secret);
            return new Response('ok', { status: 200 });
        });
        return {
            GET: ok,
            POST: f,
            PUT: f,
        };
    }
    getBody(req) {
        return new Promise((resolve) => {
            let body = '';
            req.on('data', (chunk) => {
                body += chunk;
            });
            req.on('end', () => {
                resolve(body);
            });
        });
    }
}
exports.WebhookHandler = WebhookHandler;
